/*
 *
 *  Copyright (C) 1997-2008 JDE Developers Team
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 *  Authors : David Lobato <dav.lobato@gmail.com>
 *            Antonio Pineda Cabello <apineda@gsyc.escet.urjc.es>
 *            José Antonio Santos <santoscadenas@gmail.com>
 *            Jose Maria Cañas <jmplaza@gsyc.escet.urjc.es>
 *	      Sara Marugán Alonso <s.marugan@alumnos.urjc.es>
 *
 *
 */


/**
 * jdec firewire22 driver provides video images to color variables from firewire22
 * cameras using libdc1394 library.
 *
 * This 4.3 version includes support for variable images. Only 320x240 and
 * 640x420 sizes are admitted because firewire22 cameras only support this sizes.
 *
 * @file firewire22.c
 * @author David Lobato <dlobato@gsyc.escet.urjc.es>, Antonio Pineda Cabello <apineda@gsyc.escet.urjc.es>, Jose Maria Cañas Plaza <jmplaza@gsyc.escet.urjc.es>, José Antonio Santos <santoscadenas@gmail.com> and Sara Marugán Alonso <s.marugan@alumnos.urjc.es>
 * @version 4.3
 * @date 2009-06-09
 */

#include <stdio.h>
#include <string.h>
#include <pthread.h>
#include <inttypes.h>
#include <libraw1394/raw1394.h>
#include <dc1394/control.h>
#include <interfaces/varcolor.h>
#include "jde.h"

/** Max number of cameras detected by firewire22 driver.*/
#define MAXCAM 8
/** Max number of buffers per node.*/
#define NUM_BUFFERS 1

/** Image standard number of rows*/
#define SIFNTSC_ROWS 240
/** Image standard number of columns*/
#define SIFNTSC_COLUMNS 320

/* ok the following constant should be by right included thru in Xvlib.h */
#ifndef XV_YV12
#define XV_YV12 0x32315659
#endif

#ifndef XV_YUY2
#define XV_YUY2 0x32595559
#endif

#ifndef XV_UYVY
#define XV_UYVY 0x59565955
#endif


/**
 * Color conversion functions from Bart Nabbe.
 * Corrected by Damien: bad coeficients in YUV2RGB.
 */
#define YUV2RGB(y, u, v, r, g, b)		\
  r = y + ((v*1436) >> 10);			\
  g = y - ((u*352 + v*731) >> 10);		\
  b = y + ((u*1814) >> 10);			\
  r = r < 0 ? 0 : r;				\
  g = g < 0 ? 0 : g;				\
  b = b < 0 ? 0 : b;				\
  r = r > 255 ? 255 : r;			\
  g = g > 255 ? 255 : g;			\
  b = b > 255 ? 255 : b
  
/** Automatic features for firewire22 cameras.*/
typedef struct{
  int AUTO_EXPOSURE_CONFIG;
  int AUTO_WHITE_BALANCE_CONFIG;
  int AUTO_IRIS_CONFIG;
  int AUTO_FOCUS_CONFIG;
  int AUTO_ZOOM_CONFIG;
}firewire22_features;


/* declarations for libdc1394 */
uint32_t numCameras = 0;
dc1394camera_t *cameras[MAXCAM];
dc1394featureset_t features;
dc1394video_frame_t * frames[MAXCAM];
dc1394camera_list_t * list;

/** int variable to detect when a image was captured.*/
unsigned long int lastimage=0;

/* declarations for video1394 */
/** firewire22 instant fps variable for 320X240 size.*/
int firewire22_fps320;
/** firewire22 instant resolution variable for 320X240 size.*/
int firewire22_res320;
/** firewire22 instant fps variable for 640X480 size.*/
int firewire22_fps640;
/** firewire22 instant resolution variable for 640X480 size.*/
int firewire22_res640;

/** pthread identifiers for jdec firewire22 driver threads.*/
pthread_t firewire22_th;
/** Arguments for firewire22 threads*/
int args[MAXCAM];
/** pthread state variable for jdec firewire22 driver.*/
int state;
/** pthread mutex for jdec firewire22 driver.*/
pthread_mutex_t mymutex;
/** pthread condition variable for jdec firewire22 driver.*/
pthread_cond_t condition;

/** variable to detect when the pthread is created.*/
int firewire22_thread_created;
/** variable to detect when firewire22 driver was cleaned up.*/
int firewire22_cleaned_up=0;
/** variable to detect when firewire22 driver was setup.*/
int firewire22_setup=0;
/** variable to detect when pthread must end execution.*/
int firewire22_terminate_command=0;

/* firewire22 driver API options */
/** firewire22 driver name.*/
char driver_name[256]="firewire22";
/** colors detected in config file.*/
int serve_color[MAXCAM];
/** structure to know what colors are active in the gui.*/
int color_active[MAXCAM];
/* structure to know fps from cameras */
int fps[MAXCAM];
/* structure to know id from cameras */
int id[MAXCAM];
/** feature structure from libdc1394 to set firewire22 cameras characteristics.*/
firewire22_features fwCamFeatures[MAXCAM];
/** width of each served video**/
int width[MAXCAM];
/** height of each served video**/
int height[MAXCAM];
/** id set to colorA schema.*/
int colorA_schema_id;
/** id set to colorB schema.*/
int colorB_schema_id;
/** id set to colorC schema.*/
int colorC_schema_id;
/** id set to colorD schema.*/
int colorD_schema_id;
/** id for varcolorA schema.*/
int varcolorA_schema_id;
/** id for varcolorB schema.*/
int varcolorB_schema_id;
/** id for varcolorC schema.*/
int varcolorC_schema_id;
/** id for varcolorD schema.*/
int varcolorD_schema_id;

/*API variables servidas*/
/** 'colorA' schema image data*/
char *colorA; /* sifntsc image itself */
/** 'colorA' schema clock*/
unsigned long int imageA_clock;

/** 'colorB' schema image data*/
char *colorB; /* sifntsc image itself */
/** 'colorB' schema clock*/
unsigned long int imageB_clock;

/** 'colorC' schema image data*/
char *colorC; /* sifntsc image itself */
/** 'colorC' schema clock*/
unsigned long int imageC_clock;

/** 'colorD' schema image data*/
char *colorD; /* sifntsc image itself */
/** 'colorD' schema clock*/
unsigned long int imageD_clock;

Varcolor myA,myB,myC,myD;

/*Contadores de referencias*/
/** colorA ref counter*/
int colorA_refs=0;
/** colorB ref counter*/
int colorB_refs=0;
/** colorC ref counter*/
int colorC_refs=0;
/** colorD ref counter*/
int colorD_refs=0;
/** varcolorA ref counter*/
int varcolorA_refs=0;
/** varcolorB ref counter*/
int varcolorB_refs=0;
/** varcolorC ref counter*/
int varcolorC_refs=0;
/** varcolorD ref counter*/
int varcolorD_refs=0;

/** mutex for ref counters*/
pthread_mutex_t refmutex;


/* firewire22 DRIVER FUNCTIONS */
/** function to transform a buffer from uyvy to rgb.
 *  @param src source buffer.
 *  @param dest destination buffer where the transformation will be set.
 *  @param NumPixels how many pixels per buffer.*/
void uyvy2rgb (unsigned char *src, unsigned char *dest, unsigned long long int NumPixels)
{
  register int i = (NumPixels << 1)-1;
  register int j = NumPixels + ( NumPixels << 1 ) -1;
  register int y0, y1, u, v;
  register int r, g, b;

  while (i > 0) {
    y1 = (unsigned char) src[i--];
    v  = (unsigned char) src[i--] - 128;
    y0 = (unsigned char) src[i--];
    u  = (unsigned char) src[i--] - 128;
    YUV2RGB (y1, u, v, r, g, b);
    dest[j--] = r;
    dest[j--] = g;
    dest[j--] = b;
    YUV2RGB (y0, u, v, r, g, b);
    dest[j--] = r;
    dest[j--] = g;
    dest[j--] = b;
  }
}

/** cleans up firewire22 structures and frees the firewire22 bus.*/
void cleanup(void) {
    int i;
    for (i=0; i < MAXCAM; i++) {
	if(serve_color[i]==1){
		dc1394_video_set_transmission(cameras[i], DC1394_OFF);
		dc1394_capture_stop(cameras[i]);
	}
    }
}

/** firewire22 driver closing function invoked when stopping driver.*/
void firewire22_terminate(){

  firewire22_terminate_command=1;
  cleanup();
  printf("driver firewire22 off\n");
}

/** function that will set the default configuration to all firewire22 cameras detected.*/
void set_default_firewire22_camera_config(void){

  int i;
  for(i=0; i< MAXCAM; i++){
    fwCamFeatures[i].AUTO_EXPOSURE_CONFIG=1;
    fwCamFeatures[i].AUTO_WHITE_BALANCE_CONFIG=1;
    fwCamFeatures[i].AUTO_IRIS_CONFIG=1;
    fwCamFeatures[i].AUTO_FOCUS_CONFIG=1;
    fwCamFeatures[i].AUTO_ZOOM_CONFIG=1;
  }
}

/** colorA run function following jdec platform API schemas.
 *  @param father Father id for this schema.
 *  @param brothers Brothers for this schema.
 *  @param fn arbitration function for this schema.
 *  @return integer resuming result.*/
int mycolorA_run(int father, int *brothers, arbitration fn){
   pthread_mutex_lock(&refmutex);
   if (colorA_refs>0){
      colorA_refs++;
      pthread_mutex_unlock(&refmutex);
   }
   else{
      colorA_refs=1;
      pthread_mutex_unlock(&refmutex);
      if((serve_color[0]==1)&&(color_active[0]==0)){
         color_active[0]=1;
         printf("colorA schema run (firewire22 driver)\n");
         all[colorA_schema_id].father = father;
         all[colorA_schema_id].fps = 0.;
         all[colorA_schema_id].k =0;
         all[colorA_schema_id].k =0;
         put_state(colorA_schema_id,winner);
         /* firewire22 thread goes winner */
         pthread_mutex_lock(&mymutex);
         state=winner;
         pthread_cond_signal(&condition);
         pthread_mutex_unlock(&mymutex);
      }
   }
   return 0;
}

/** colorA stop function following jdec platform API schemas.
 *  @return integer stopping result.*/
int mycolorA_stop(){

   pthread_mutex_lock(&refmutex);
   if (colorA_refs>1){
      colorA_refs--;
      pthread_mutex_unlock(&refmutex);
   }
   else{
      colorA_refs=0;
      pthread_mutex_unlock(&refmutex);
      if((serve_color[0]==1)&&(color_active[0]==1)){
         color_active[0]=0;
         put_state(colorA_schema_id,slept);
         printf("colorA schema stop (firewire22 driver)\n");
      }
   }
   return 0;
}

/** colorB run function following jdec platform API schemas.
 *  @param father Father id for this schema.
 *  @param brothers Brothers for this schema.
 *  @param fn arbitration function for this schema.
 *  @return integer resuming result.*/
int mycolorB_run(int father, int *brothers, arbitration fn){

   pthread_mutex_lock(&refmutex);
   if (colorB_refs>0){
      colorB_refs++;
      pthread_mutex_unlock(&refmutex);
   }
   else{
      colorB_refs=1;
      pthread_mutex_unlock(&refmutex);
      if((serve_color[1]==1)&&(color_active[1]==0)){
         color_active[1]=1;
         printf("colorB schema run (firewire22 driver)\n");
         all[colorB_schema_id].father = father;
         all[colorB_schema_id].fps = 0.;
         all[colorB_schema_id].k =0;
   
         put_state(colorB_schema_id,winner);

         /* firewire22 thread goes winner */
         pthread_mutex_lock(&mymutex);
         state=winner;
         pthread_cond_signal(&condition);
         pthread_mutex_unlock(&mymutex);
      }
   }
   return 0;
}

/** colorB stop function following jdec platform API schemas.
 *  @return integer stopping result.*/
int mycolorB_stop(){

   pthread_mutex_lock(&refmutex);
   if (colorB_refs>1){
      colorB_refs--;
      pthread_mutex_unlock(&refmutex);
   }
   else{
      colorB_refs=0;
      pthread_mutex_unlock(&refmutex);
      if((serve_color[1]==1)&&(color_active[1]==1)){
         color_active[1]=0;
         printf("colorB schema stop (firewire22 driver)\n");
         put_state(colorB_schema_id,slept);
      }
   }
   return 0;
}

/** colorC run function following jdec platform API schemas.
 *  @param father Father id for this schema.
 *  @param brothers Brothers for this schema.
 *  @param fn arbitration function for this schema.
 *  @return integer resuming result.*/
int mycolorC_run(int father, int *brothers, arbitration fn){

   pthread_mutex_lock(&refmutex);
   if (colorC_refs>0){
      colorC_refs++;
      pthread_mutex_unlock(&refmutex);
   }
   else{
      colorC_refs=1;
      pthread_mutex_unlock(&refmutex);
      if((serve_color[2]==1)&&(color_active[2]==0)){
         color_active[2]=1;
         printf("colorC schema run (firewire22 driver)\n");
         all[colorC_schema_id].father = father;
         all[colorC_schema_id].fps = 0.;
         all[colorC_schema_id].k =0;
         put_state(colorC_schema_id,winner);
         /* firewire22 thread goes winner */
         pthread_mutex_lock(&mymutex);
         state=winner;
         pthread_cond_signal(&condition);
         pthread_mutex_unlock(&mymutex);
      }
   }
   return 0;
}

/** colorC stop function following jdec platform API schemas.
 *  @return integer stopping result.*/
int mycolorC_stop(){
   
   pthread_mutex_lock(&refmutex);
   if (colorC_refs>1){
      colorC_refs--;
      pthread_mutex_unlock(&refmutex);
   }
   else{
      colorC_refs=0;
      pthread_mutex_unlock(&refmutex);
      if((serve_color[2]==1)&&(color_active[2]==1)){
         color_active[2]=0;
         printf("colorC schema stop (firewire22 driver)\n");
         put_state(colorC_schema_id,slept);
      }
   }
   return 0;
}

/** colorD run function following jdec platform API schemas.
 *  @param father Father id for this schema.
 *  @param brothers Brothers for this schema.
 *  @param fn arbitration function for this schema.
 *  @return integer resuming result.*/
int mycolorD_run(int father, int *brothers, arbitration fn){

   pthread_mutex_lock(&refmutex);
   if (colorD_refs>0){
      colorD_refs++;
      pthread_mutex_unlock(&refmutex);
   }
   else{
      colorD_refs=1;
      pthread_mutex_unlock(&refmutex);
      if((serve_color[3]==1)&&(color_active[3]==0)){
         color_active[3]=1;
         printf("colorD schema run (firewire22 driver)\n");
         all[colorD_schema_id].father = father;
         all[colorD_schema_id].fps = 0.;
         all[colorD_schema_id].k =0;
         put_state(colorD_schema_id,winner);
         /* firewire22 thread goes winner */
         pthread_mutex_lock(&mymutex);
         state=winner;
         pthread_cond_signal(&condition);
         pthread_mutex_unlock(&mymutex);
      }
   }
   return 0;
}

/** colorD stop function following jdec platform API schemas.
 *  @return integer stopping result.*/
int mycolorD_stop(){
   
   pthread_mutex_lock(&refmutex);
   if (colorD_refs>1){
      colorD_refs--;
      pthread_mutex_unlock(&refmutex);
   }
   else{
      colorD_refs=0;
      pthread_mutex_unlock(&refmutex);
      if((serve_color[3]==1)&&(color_active[3]==1)){
         color_active[3]=0;
         printf("colorD schema stop (firewire22 driver)\n");
         put_state(colorD_schema_id,slept);
      }
   }
   return 0;
}

/** varcolorA run function following jdec platform API schemas.
 *  @param father Father id for this schema.
 *  @param brothers Brothers for this schema.
 *  @param fn arbitration function for this schema.
 *  @return integer resuming result.*/
int myvarcolorA_run(int father, int *brothers, arbitration fn){
   pthread_mutex_lock(&refmutex);
   if (varcolorA_refs>0){
      varcolorA_refs++;
      pthread_mutex_unlock(&refmutex);
   }
   else{
      varcolorA_refs=1;
      pthread_mutex_unlock(&refmutex);
      if((serve_color[4]==1)&&(color_active[4]==0)){
         color_active[4]=1;
         printf("varcolorA schema run (firewire22 driver)\n");
         all[varcolorA_schema_id].father = father;
         all[varcolorA_schema_id].fps = 0.;
         all[varcolorA_schema_id].k =0;
         put_state(varcolorA_schema_id,winner);
         /* firewire22 thread goes winner */
         pthread_mutex_lock(&mymutex);
         state=winner;
         pthread_cond_signal(&condition);
         pthread_mutex_unlock(&mymutex);
      }
   }
   return 0;
}

/** varcolorA stop function following jdec platform API schemas.
 *  @return integer stopping result.*/
int myvarcolorA_stop(){
   pthread_mutex_lock(&refmutex);
   if (varcolorA_refs>1){
      varcolorA_refs--;
      pthread_mutex_unlock(&refmutex);
   }
   else{
      varcolorA_refs=0;
      pthread_mutex_unlock(&refmutex);
      if((serve_color[4]==1)&&(color_active[4]==1)){
         color_active[4]=0;
         printf("varcolorA schema stop (firewire22 driver)\n");
         put_state(varcolorA_schema_id,slept);
      }
   }
   return 0;
}

/** varcolorB run function following jdec platform API schemas.
 *  @param father Father id for this schema.
 *  @param brothers Brothers for this schema.
 *  @param fn arbitration function for this schema.
 *  @return integer resuming result.*/
int myvarcolorB_run(int father, int *brothers, arbitration fn){
   pthread_mutex_lock(&refmutex);
   if (varcolorB_refs>0){
      varcolorB_refs++;
      pthread_mutex_unlock(&refmutex);
   }
   else{
      varcolorB_refs=1;
      pthread_mutex_unlock(&refmutex);
      if((serve_color[5]==1)&&(color_active[5]==0)){
         color_active[5]=1;
         printf("varcolorB schema run (firewire22 driver)\n");
         all[varcolorB_schema_id].father = father;
         all[varcolorB_schema_id].fps = 0.;
         all[varcolorB_schema_id].k =0;
         put_state(varcolorB_schema_id,winner);
         /* firewire22 thread goes winner */
         pthread_mutex_lock(&mymutex);
         state=winner;
         pthread_cond_signal(&condition);
         pthread_mutex_unlock(&mymutex);
      }
   }
   return 0;
}

/** varcolorB stop function following jdec platform API schemas.
 *  @return integer stopping result.*/
int myvarcolorB_stop(){
   pthread_mutex_lock(&refmutex);
   if (varcolorB_refs>1){
      varcolorB_refs--;
      pthread_mutex_unlock(&refmutex);
   }
   else{
      varcolorB_refs=0;
      pthread_mutex_unlock(&refmutex);
      if((serve_color[5]==1)&&(color_active[5]==1)){
         color_active[5]=0;
         printf("varcolorB schema stop (firewire22 driver)\n");
         put_state(varcolorB_schema_id,slept);
      }
   }
   return 0;
}

/** varcolorC run function following jdec platform API schemas.
 *  @param father Father id for this schema.
 *  @param brothers Brothers for this schema.
 *  @param fn arbitration function for this schema.
 *  @return integer resuming result.*/
int myvarcolorC_run(int father, int *brothers, arbitration fn){
   pthread_mutex_lock(&refmutex);
   if (varcolorC_refs>0){
      varcolorC_refs++;
      pthread_mutex_unlock(&refmutex);
   }
   else{
      varcolorC_refs=1;
      pthread_mutex_unlock(&refmutex);
      if((serve_color[6]==1)&&(color_active[6]==0)){
         color_active[6]=1;
         printf("varcolorC schema run (firewire22 driver)\n");
         all[varcolorC_schema_id].father = father;
         all[varcolorC_schema_id].fps = 0.;
         all[varcolorC_schema_id].k =0;
         put_state(varcolorC_schema_id,winner);
         /* firewire22 thread goes winner */
         pthread_mutex_lock(&mymutex);
         state=winner;
         pthread_cond_signal(&condition);
         pthread_mutex_unlock(&mymutex);
      }
   }
   return 0;
}

/** varcolorC stop function following jdec platform API schemas.
 *  @return integer stopping result.*/
int myvarcolorC_stop(){
   pthread_mutex_lock(&refmutex);
   if (varcolorC_refs>1){
      varcolorC_refs--;
      pthread_mutex_unlock(&refmutex);
   }
   else{
      varcolorC_refs=0;
      pthread_mutex_unlock(&refmutex);
      if((serve_color[6]==1)&&(color_active[6]==1)){
         color_active[6]=0;
         printf("varcolorC schema stop (firewire22 driver)\n");
         put_state(varcolorC_schema_id,slept);
      }
   }
   return 0;
}

/** varcolorD run function following jdec platform API schemas.
 *  @param father Father id for this schema.
 *  @param brothers Brothers for this schema.
 *  @param fn arbitration function for this schema.
 *  @return integer resuming result.*/
int myvarcolorD_run(int father, int *brothers, arbitration fn){
   pthread_mutex_lock(&refmutex);
   if (varcolorD_refs>0){
      varcolorD_refs++;
      pthread_mutex_unlock(&refmutex);
   }
   else{
      varcolorD_refs=1;
      pthread_mutex_unlock(&refmutex);
      if((serve_color[7]==1)&&(color_active[7]==0)){
         color_active[7]=1;
         printf("varcolorD schema run (firewire22 driver)\n");
         all[varcolorD_schema_id].father = father;
         all[varcolorD_schema_id].fps = 0.;
         all[varcolorD_schema_id].k =0;
         put_state(varcolorD_schema_id,winner);
         /* firewire22 thread goes winner */
         pthread_mutex_lock(&mymutex);
         state=winner;
         pthread_cond_signal(&condition);
         pthread_mutex_unlock(&mymutex);
      }
   }
   return 0;
}

/** varcolorD stop function following jdec platform API schemas.
 *  @return integer stopping result.*/
int myvarcolorD_stop(){
   pthread_mutex_lock(&refmutex);
   if (varcolorD_refs>1){
      varcolorD_refs--;
      pthread_mutex_unlock(&refmutex);
   }
   else{
      varcolorD_refs=0;
      pthread_mutex_unlock(&refmutex);
      if((serve_color[7]==1)&&(color_active[7]==1)){
         color_active[7]=0;
         printf("varcolorD schema stop (firewire22 driver)\n");
         put_state(varcolorD_schema_id,slept);
      }
   }
   return 0;
}


/** firewire22 driver pthread function.*/
void *firewire22_thread(void *id)
{
   int j;
   int captured[MAXCAM];
   int cams;
   
  printf("firewire22 driver started up\n");

  do{
        
    pthread_mutex_lock(&mymutex);

    if (state==slept){
      printf("firewire22 thread in sleep mode\n");
      pthread_cond_wait(&condition,&mymutex);
      printf("firewire22 thread woke up\n");
      pthread_mutex_unlock(&mymutex);
      
    }else{
        pthread_mutex_unlock(&mymutex);
       /* image capture for all cameras - only capturing when needed */

       for (j = 0; j < MAXCAM; j++) {
		//printf("# serve color %d is %d\n",j,cameras[j]);
       		if (serve_color[j]==1)
       		{       		 
             		if (dc1394_capture_dequeue(cameras[j], DC1394_CAPTURE_POLICY_WAIT, &frames[j])!=DC1394_SUCCESS)
                		dc1394_log_error("Failed to capture from camera %d", j);    
			else
				captured[j]=TRUE;
       		}
       }
       
       if (serve_color[0]==1 && captured[0]==TRUE && color_active[0]==1)
       {
        	speedcounter(colorA_schema_id);
        	uyvy2rgb((unsigned char*)frames[0]->image,(unsigned char*)colorA,frames[0]->size[0]*frames[0]->size[1]);
		imageA_clock=lastimage;
		captured[0]=FALSE;
       }
       if (serve_color[1]==1 && captured[1]==TRUE && color_active[1]==1)	
       {
       		speedcounter(colorB_schema_id);
       		uyvy2rgb((unsigned char*)frames[1]->image,(unsigned char*)colorB,frames[1]->size[0]*frames[1]->size[1]);
		imageB_clock=lastimage;
		captured[1]=FALSE;
       }
       if (serve_color[2]==1 && captured[2]==TRUE && color_active[2]==1)
       {
        	speedcounter(colorC_schema_id);
        	uyvy2rgb((unsigned char*)frames[2]->image,(unsigned char*)colorC,frames[2]->size[0]*frames[2]->size[1]);
		imageC_clock=lastimage;
		captured[2]=FALSE;
       }
       if (serve_color[3]==1 && captured[3]==TRUE && color_active[3]==1)	
       {
       		speedcounter(colorD_schema_id);
       		uyvy2rgb((unsigned char*)frames[3]->image,(unsigned char*)colorD,frames[3]->size[0]*frames[3]->size[1]);
		imageD_clock=lastimage;
		captured[3]=FALSE;
       }

       if (serve_color[4]==1 && captured[4]==TRUE && color_active[4]==1)
       {
        	speedcounter(varcolorA_schema_id);
        	uyvy2rgb((unsigned char*)frames[4]->image,(unsigned char*)myA.img,frames[4]->size[0]*frames[4]->size[1]);
		myA.clock=lastimage;
		captured[4]=FALSE;
       }
       if (serve_color[5]==1 && captured[5]==TRUE && color_active[5]==1)
       {
        	speedcounter(varcolorB_schema_id);
        	uyvy2rgb((unsigned char*)frames[5]->image,(unsigned char*)myB.img,frames[5]->size[0]*frames[5]->size[1]);
		myB.clock=lastimage;
		captured[5]=FALSE;
       }
       if (serve_color[6]==1 && captured[6]==TRUE && color_active[6]==1)
       {
        	speedcounter(varcolorC_schema_id);
        	uyvy2rgb((unsigned char*)frames[6]->image,(unsigned char*)myC.img,frames[6]->size[0]*frames[6]->size[1]);
		myC.clock=lastimage;
		captured[6]=FALSE;
       }
       if (serve_color[7]==1 && captured[7]==TRUE && color_active[7]==1)
       {
        	speedcounter(varcolorD_schema_id);
        	uyvy2rgb((unsigned char*)frames[7]->image,(unsigned char*)myD.img,frames[7]->size[0]*frames[7]->size[1]);
		myD.clock=lastimage;
		captured[7]=FALSE;
       }
     
       cams=0;
       for (j = 0; j < MAXCAM; j++) {
           if (serve_color[j]==1 && frames[j])
               dc1394_capture_enqueue (cameras[j], frames[j]);

	   if(color_active[j]==1)
		cams++;
       }

       /* detects if there aren't any cameras and goes to slept */
       if(cams==0){
		state=slept;
       }

       lastimage++;

      }   
  }while(firewire22_terminate_command==0);

  pthread_exit(0);
}

/** Determines if an image configuration could be captured or not
 *  @param width width of the possible image
 *  @param height height of the possible image
 *  @return 1 if the size could be captured, otherwise 0
 */
int size_ok(int width, int height){
   if( (width==320 && height==240) || (width==640 && height==480))
      return 1;
   else
      return 0;
}


/** firewire22 driver parse configuration file function.
 *  @param configfile path and name to the config file.
 *  @return 0 if parsing was successful or -1 if something went wrong.*/
int firewire22_parseconf(char *configfile){

   int end_parse=0; int end_section=0; int driver_config_parsed=0;
   FILE *myfile;
   const int limit = 256;

   if ((myfile=fopen(configfile,"r"))==NULL){
      printf("firewire22: cannot find config file\n");
      return -1;
   }

   do{
    
      char word[256],word2[256],buffer_file[256];
      int i=0; int j=0;

      buffer_file[0]=fgetc(myfile);
    
      /* end of file */
      if (feof(myfile)){
         end_section=1;
         end_parse=1;
      
         /* line comment */
      }else if (buffer_file[0]=='#') {
         while(fgetc(myfile)!='\n');
      
         /* white spaces */
      }else if (buffer_file[0]==' ') {
         while(buffer_file[0]==' ') buffer_file[0]=fgetc(myfile);

         /* tabs */
      }else if(buffer_file[0]=='\t') {
         while(buffer_file[0]=='\t') buffer_file[0]=fgetc(myfile);
         /* storing line in buffer */
      }else{
      
         while(buffer_file[i]!='\n') buffer_file[++i]=fgetc(myfile);
         buffer_file[++i]='\0';

         if (i >= limit-1) {
            printf("%s...\n", buffer_file);
            printf ("firewire22: line too long in config file!\n");
            jdeshutdown(-1);
         }
      
         /* first word of the line */
         if (sscanf(buffer_file,"%s",word)==1){
            if (strcmp(word,"driver")==0) {
               while((buffer_file[j]!='\n')&&(buffer_file[j]!=' ')&&(buffer_file[j]!='\0')&&(buffer_file[j]!='\t')) j++;
               sscanf(&buffer_file[j],"%s",word2);
	  
               /* checking if this section matchs our driver name */
               if (strcmp(word2,driver_name)==0){
                  /* the sections match */
                  do{
	      
                     char buffer_file2[256],word3[256],word4[256];
                     char word5[256],word6[256], word7[256],word8[256],word9[256];
                     int k=0; int z=0;

                     buffer_file2[0]=fgetc(myfile);
	      
                     /* end of file */
                     if (feof(myfile)){
                        end_section=1;
                        end_parse=1;
		
                        /* line comment */
                     }else if (buffer_file2[0]=='#') {
                        while(fgetc(myfile)!='\n');
	      
                        /* white spaces */
                     }else if (buffer_file2[0]==' ') {
                        while(buffer_file2[0]==' ') buffer_file2[0]=fgetc(myfile);

                        /* tabs */
                     }else if(buffer_file2[0]=='\t') {
                        while(buffer_file2[0]=='\t') buffer_file2[0]=fgetc(myfile);

                        /* storing line in buffer */
                     }else{
		
                        while(buffer_file2[k]!='\n') buffer_file2[++k]=fgetc(myfile);
                        buffer_file2[++k]='\0';
		
                        /* first word of the line */
                        if (sscanf(buffer_file2,"%s",word3)==1){
                           if (strcmp(word3,"end_driver")==0) {
                              while((buffer_file2[z]!='\n')&&(buffer_file2[z]!=' ')&&(buffer_file2[z]!='\0')&&(buffer_file2[z]!='\t')) z++;
                              driver_config_parsed=1;
                              end_section=1;
                              end_parse=1;
		    
                           }else if (strcmp(word3,"driver")==0) {
                              while((buffer_file2[z]!='\n')&&
                                     (buffer_file2[z]!=' ')&&
                                     (buffer_file2[z]!='\0')&&
                                     (buffer_file2[z]!='\t')) z++;
                              printf("firewire22: error in config file.\n'end_section' keyword required before starting new driver section.\n");
                              end_section=1; end_parse=1;

                           }else if(strcmp(word3,"provides")==0){
                              int words;
                              while((buffer_file2[z]!='\n')&&
                                     (buffer_file2[z]!=' ')&&
                                     (buffer_file2[z]!='\0')&&
                                     (buffer_file2[z]!='\t')) z++;
                              words=sscanf(buffer_file2,"%s %s %s %s %s %s %s",
                                           word3,word4,word5,word6,word7,word8,word9);
                              if(words==5){
                                 if(strcmp(word4,"colorA")==0){
                                    serve_color[0]=1;
				    id[0]=atoi(word5);
				    fps[0]=atoi(word6);
                                    width[0] = SIFNTSC_COLUMNS;
                                    height[0] = SIFNTSC_ROWS;
                                    if(strcmp(word7,"autofocus_on")!=0)
                                          fwCamFeatures[0].AUTO_FOCUS_CONFIG=0;
                                    else
                                          fwCamFeatures[0].AUTO_FOCUS_CONFIG=1;

                                 }else if(strcmp(word4,"colorB")==0){
                                    serve_color[1]=1;
				    id[1]=atoi(word5);
				    fps[1]=atoi(word6);
                                    width[1] = SIFNTSC_COLUMNS;
                                    height[1] = SIFNTSC_ROWS;
                                    if(strcmp(word7,"autofocus_on")!=0)
                                          fwCamFeatures[1].AUTO_FOCUS_CONFIG=0;
                                    else
                                          fwCamFeatures[1].AUTO_FOCUS_CONFIG=1;

                                 }else if(strcmp(word4,"colorC")==0){
                                    serve_color[2]=1;
				    id[2]=atoi(word5);
				    fps[2]=atoi(word6);
                                    width[2] = SIFNTSC_COLUMNS;
                                    height[2] = SIFNTSC_ROWS;
                                    if(strcmp(word7,"autofocus_on")!=0)
                                          fwCamFeatures[2].AUTO_FOCUS_CONFIG=0;
                                    else
                                          fwCamFeatures[2].AUTO_FOCUS_CONFIG=1;

                                 }else if(strcmp(word4,"colorD")==0){
                                    serve_color[3]=1;
				    id[3]=atoi(word5);
				    fps[3]=atoi(word6);
                                    width[3] = SIFNTSC_COLUMNS;
                                    height[3] = SIFNTSC_ROWS;
                                    if(strcmp(word7,"autofocus_on")!=0)
                                          fwCamFeatures[3].AUTO_FOCUS_CONFIG=0;
                                    else
                                          fwCamFeatures[3].AUTO_FOCUS_CONFIG=1;
                                 }
                              }
                              else if (words==7){
                                 if(strcmp(word4,"varcolorA")==0){
                                    serve_color[4]=1;
				    id[4]=atoi(word5);
				    fps[4]=atoi(word6);
                                    width[4] = atoi(word7);
                                    height[4] = atoi(word8);
                                    if (!size_ok(width[4], height[4])){
                                          fprintf (stderr, "firewire22: Wrong image size for varcolorA, changed to default size 320X240\n");
                                          width[4] = SIFNTSC_COLUMNS;
                                          height[4] = SIFNTSC_ROWS;
				    }
                                       
                                    if(strcmp(word9,"autofocus_on")!=0)
                                          fwCamFeatures[4].AUTO_FOCUS_CONFIG=0;
                                    else
                                          fwCamFeatures[4].AUTO_FOCUS_CONFIG=1;
                                 }else if(strcmp(word4,"varcolorB")==0){
                                    serve_color[5]=1;
				    id[5]=atoi(word5);
				    fps[5]=atoi(word6);
                                    width[5] = atoi(word7);
                                    height[5] = atoi(word8);
                                    if (!size_ok(width[5], height[5])){
                                          fprintf (stderr, "firewire22: Wrong image size for varcolorB, changed to default size 320X240\n");
                                          width[5] = SIFNTSC_COLUMNS;
                                          height[5] = SIFNTSC_ROWS;
				    }
                                       
                                    if(strcmp(word9,"autofocus_on")!=0)
                                          fwCamFeatures[5].AUTO_FOCUS_CONFIG=0;
                                    else
                                          fwCamFeatures[5].AUTO_FOCUS_CONFIG=1;

                                 }else if(strcmp(word4,"varcolorC")==0){
                                    serve_color[6]=1;
				    id[6]=atoi(word5);
				    fps[6]=atoi(word6);
                                    width[6] = atoi(word7);
                                    height[6] = atoi(word8);
                                    if (!size_ok(width[6], height[6])){
                                          fprintf (stderr, "firewire22: Wrong image size for varcolorC, changed to default size 320X240\n");
                                          width[6] = SIFNTSC_COLUMNS;
                                          height[6] = SIFNTSC_ROWS;
				    }
                                       
                                    if(strcmp(word9,"autofocus_on")!=0)
                                          fwCamFeatures[6].AUTO_FOCUS_CONFIG=0;
                                    else
                                          fwCamFeatures[6].AUTO_FOCUS_CONFIG=1;

                                 }else if(strcmp(word4,"varcolorD")==0){
                                    serve_color[7]=1;
				    id[7]=atoi(word5);
				    fps[7]=atoi(word6);
                                    width[7] = atoi(word7);
                                    height[7] = atoi(word8);
                                    if (!size_ok(width[7], height[7])){
                                          fprintf (stderr, "firewire22: Wrong image size for varcolorD, changed to default size 320X240\n");
                                          width[7] = SIFNTSC_COLUMNS;
                                          height[7] = SIFNTSC_ROWS;
				    }
                                       
                                    if(strcmp(word9,"autofocus_on")!=0)
                                          fwCamFeatures[7].AUTO_FOCUS_CONFIG=0;
                                    else
                                          fwCamFeatures[7].AUTO_FOCUS_CONFIG=1;
                                 }

                              }else{
                                 printf("firewire22: provides line incorrect\n");
                              }
                           }else printf("firewire22: i don't know what to do with '%s'\n",buffer_file2);
                        }
                     }
                  }while(end_section==0);
                  end_section=0;
               }
            }
         }
      }
   }while(end_parse==0);
  
   /* checking if a driver section was read */
   if(driver_config_parsed==1){
      if((serve_color[0]==0)&&(serve_color[1]==0)&&(serve_color[2]==0)&&(serve_color[3]==0)&&
          (serve_color[4]==0)&&(serve_color[5]==0)&&(serve_color[6]==0)&&(serve_color[7]==0)){
         printf("firewire22: warning! no color provided.\n");
      }
      return 0;
   }else return -1;
}

/** firewire22 driver init function. It will start all firewire22 required devices
 *  and setting them the default configuration.
 *  @return 0 if initialitation was successful or -1 if something went wrong.*/
void firewire22_deviceinit(){
	int p,i,j;
	dc1394_t * d;  
	dc1394error_t err;
        long format=0;
        int res;
        float f;
	int used[MAXCAM];

    	format=XV_UYVY;
   
	list=NULL;
	d = dc1394_new ();
	err=dc1394_camera_enumerate (d, &list);

	if (list == NULL) {
		perror("can't access device\n");
		exit(-1);
	}
  
	numCameras = 0;
	memset(used,0,sizeof(int)*MAXCAM);
	for(i=0; i<MAXCAM; i++){
		if(serve_color[i]==1 && id[i]>=0){
			if(id[i]<list->num && used[id[i]]==0){
				//printf("# camara %d tiene id %d\n",i,id[i]);
				cameras[i] = dc1394_camera_new (d, list->ids[id[i]].guid);
				if (!cameras[i]){
		    			dc1394_log_warning("Failed to initialize camera with guid %llx", list->ids[id[i]].guid);
					serve_color[i]=0;
					continue;
				}
				used[id[i]]=1;
				numCameras++;

				if(width[i]==320)
					res = DC1394_VIDEO_MODE_320x240_YUV422;
				else
					res = DC1394_VIDEO_MODE_640x480_YUV422;

				if(fps[i]==30){
					f = DC1394_FRAMERATE_30;
					printf("# pongo a 30\n");
				}
				else if(fps[i]==15){
					f = DC1394_FRAMERATE_15;
					printf("# pongo a 15\n");
				}
				else{
					f = DC1394_FRAMERATE_15;
					fprintf(stderr,"firewire22: Wrong framerate for camera %d, set to default 15 fps\n",id[i]);
				}

				err=dc1394_video_set_iso_speed(cameras[i], DC1394_ISO_SPEED_400);
				DC1394_ERR_CLN_RTN(err,cleanup(),"Could not set ISO speed");

				err=dc1394_video_set_mode(cameras[i], res);
				DC1394_ERR_CLN_RTN(err,cleanup(),"Could not set video mode");

				err=dc1394_video_set_framerate(cameras[i], f);
				DC1394_ERR_CLN_RTN(err,cleanup(),"Could not set framerate");

				err=dc1394_capture_setup(cameras[i],NUM_BUFFERS, DC1394_CAPTURE_FLAGS_DEFAULT);
				DC1394_ERR_CLN_RTN(err,cleanup(),"Could not setup camera-\nmake sure that the video mode and framerate are\nsupported by your camera");

				err=dc1394_video_set_transmission(cameras[i], DC1394_ON);
				DC1394_ERR_CLN_RTN(err,cleanup(),"Could not start camera iso transmission");
			}
			else{
				fprintf(stderr,"firewire22 ERROR: incorrect id for camera %d\n",i);
				serve_color[i]=0;
			}
		}
        }
	
	printf ("firewire22: cams found: %d\n",numCameras);
    	dc1394_camera_free_list (list);  
  
	if (numCameras < 1) {
		perror("no cameras found\n");		
		exit(-1);
	}

	firewire22_setup=1;
}

/** firewire22 driver init function following jdec platform API for drivers.
 *  @param configfile path and name to the config file of this driver.*/
void firewire22_init(char *configfile)
{
  int i;

  /* reseting serve color array and setting default options */
  for(i=0;i<MAXCAM;i++){
     serve_color[i]=0;
     color_active[i]=0;
     id[i]=-1;
  }
  //set_default_firewire22_camera_config();


  /* we call the function to parse the config file */
  if(firewire22_parseconf(configfile)==-1){
    printf("firewire22: cannot initiate driver. configfile parsing error.\n");
    exit(-1);
  }

  /* firewire22 initialitation */
  if(firewire22_setup==0) firewire22_deviceinit();

  if (firewire22_thread_created!=1)
  {
  	pthread_mutex_lock(&mymutex);
  	state=slept;
  	args[0]=0;
  	pthread_create(&firewire22_th,NULL,firewire22_thread,&args[0]);
  	firewire22_thread_created=1;
  	pthread_mutex_unlock(&mymutex);
  }

  /* displays autofocus message for each camera */
  if(serve_color[0]==1){
      all[num_schemas].id = (int *) &colorA_schema_id;
      strcpy(all[num_schemas].name,"colorA");
      all[num_schemas].run = (runFn) mycolorA_run;
      all[num_schemas].stop = (stopFn) mycolorA_stop;
      printf("%s schema loaded (id %d)\n",all[num_schemas].name,num_schemas);
      (*(all[num_schemas].id)) = num_schemas;
      all[num_schemas].fps = 0.;
      all[num_schemas].k =0;
      all[num_schemas].state=slept;
      all[num_schemas].terminate = NULL;
      all[num_schemas].handle = NULL;
      num_schemas++;
      myexport("colorA","id",&colorA_schema_id);
      myexport("colorA","colorA",&colorA);
      myexport("colorA","clock", &imageA_clock);
      myexport("colorA","width",&width[0]);
      myexport("colorA","height",&height[0]);
      myexport("colorA","run",(void *)mycolorA_run);
      myexport("colorA","stop",(void *)mycolorA_stop);

      colorA=(char *)malloc (width[0]*height[0]*3);
  }

  if(serve_color[1]==1){
      all[num_schemas].id = (int *) &colorB_schema_id;
      strcpy(all[num_schemas].name,"colorB");
      all[num_schemas].run = (runFn) mycolorB_run;
      all[num_schemas].stop = (stopFn) mycolorB_stop;
      printf("%s schema loaded (id %d)\n",all[num_schemas].name,num_schemas);
      (*(all[num_schemas].id)) = num_schemas;
      all[num_schemas].fps = 0.;
      all[num_schemas].k =0;
      all[num_schemas].state=slept;
      all[num_schemas].terminate = NULL;
      all[num_schemas].handle = NULL;
      num_schemas++;
      myexport("colorB","id",&colorB_schema_id);
      myexport("colorB","colorB",&colorB);
      myexport("colorB","clock", &imageB_clock);
      myexport("colorB","width",&width[1]);
      myexport("colorB","height",&height[1]);
      myexport("colorB","run",(void *)mycolorB_run);
      myexport("colorB","stop",(void *)mycolorB_stop);

      colorB=(char *)malloc (width[1]*height[1]*3);
  }

  if(serve_color[2]==1){
      all[num_schemas].id = (int *) &colorC_schema_id;
      strcpy(all[num_schemas].name,"colorC");
      all[num_schemas].run = (runFn) mycolorC_run;
      all[num_schemas].stop = (stopFn) mycolorC_stop;
      printf("%s schema loaded (id %d)\n",all[num_schemas].name,num_schemas);
      (*(all[num_schemas].id)) = num_schemas;
      all[num_schemas].fps = 0.;
      all[num_schemas].k =0;
      all[num_schemas].state=slept;
      all[num_schemas].terminate = NULL;
      all[num_schemas].handle = NULL;
      num_schemas++;
      myexport("colorC","id",&colorC_schema_id);
      myexport("colorC","colorC",&colorC);
      myexport("colorC","clock", &imageC_clock);
      myexport("colorC","width",&width[2]);
      myexport("colorC","height",&height[2]);
      myexport("colorC","run",(void *)mycolorC_run);
      myexport("colorC","stop",(void *)mycolorC_stop);

      colorC=(char *)malloc (width[2]*height[2]*3);
  }

  if(serve_color[3]==1){
      all[num_schemas].id = (int *) &colorD_schema_id;
      strcpy(all[num_schemas].name,"colorD");
      all[num_schemas].run = (runFn) mycolorD_run;
      all[num_schemas].stop = (stopFn) mycolorD_stop;
      printf("%s schema loaded (id %d)\n",all[num_schemas].name,num_schemas);
      (*(all[num_schemas].id)) = num_schemas;
      all[num_schemas].fps = 0.;
      all[num_schemas].k =0;
      all[num_schemas].state=slept;
      all[num_schemas].terminate = NULL;
      all[num_schemas].handle = NULL;
      num_schemas++;
      myexport("colorD","id",&colorD_schema_id);
      myexport("colorD","colorD",&colorD);
      myexport("colorD","clock", &imageD_clock);
      myexport("colorD","width",&width[3]);
      myexport("colorD","height",&height[3]);
      myexport("colorD","run",(void *)mycolorD_run);
      myexport("colorD","stop",(void *)mycolorD_stop);

      colorD=(char *)malloc (width[3]*height[3]*3);
  }
  /*creates new schema for varcolorA*/
  if(serve_color[4]==1){
        all[num_schemas].id = (int *) &varcolorA_schema_id;
        strcpy(all[num_schemas].name,"varcolorA");
        all[num_schemas].run = (runFn) myvarcolorA_run;
        all[num_schemas].stop = (stopFn) myvarcolorA_stop;
        printf("%s schema loaded (id %d)\n",all[num_schemas].name,num_schemas);
        (*(all[num_schemas].id)) = num_schemas;
        all[num_schemas].fps = 0.;
        all[num_schemas].k =0;
        all[num_schemas].state=slept;
        all[num_schemas].terminate = NULL;
        all[num_schemas].handle = NULL;
        num_schemas++;
	
	myA.img=(unsigned char *)malloc (width[4]*height[4]*3);
	myA.width=width[4];
	myA.height=height[4];
	myA.clock=0;
        myexport("varcolorA","varcolorA",&myA);
        myexport("varcolorA","id",&varcolorA_schema_id);
        myexport("varcolorA","run",(void *)myvarcolorA_run);
        myexport("varcolorA","stop",(void *)myvarcolorA_stop);
  }
  /*creates new schema for varcolorB*/
  if(serve_color[5]==1){
        all[num_schemas].id = (int *) &varcolorB_schema_id;
        strcpy(all[num_schemas].name,"varcolorB");
        all[num_schemas].run = (runFn) myvarcolorB_run;
        all[num_schemas].stop = (stopFn) myvarcolorB_stop;
        printf("%s schema loaded (id %d)\n",all[num_schemas].name,num_schemas);
        (*(all[num_schemas].id)) = num_schemas;
        all[num_schemas].fps = 0.;
        all[num_schemas].k =0;
        all[num_schemas].state=slept;
        all[num_schemas].terminate = NULL;
        all[num_schemas].handle = NULL;
        num_schemas++;

        myB.img=(char *)malloc (width[5]*height[5]*3);
	myB.width=width[5];
	myB.height=height[5];
	myB.clock=0;
        myexport("varcolorB","varcolorB",&myB);
        myexport("varcolorB","id",&varcolorB_schema_id);
        myexport("varcolorB","run",(void *)myvarcolorB_run);
        myexport("varcolorB","stop",(void *)myvarcolorB_stop);
  }

  /*creates new schema for varcolorC*/
  if(serve_color[6]==1){
        all[num_schemas].id = (int *) &varcolorC_schema_id;
        strcpy(all[num_schemas].name,"varcolorC");
        all[num_schemas].run = (runFn) myvarcolorC_run;
        all[num_schemas].stop = (stopFn) myvarcolorC_stop;
        printf("%s schema loaded (id %d)\n",all[num_schemas].name,num_schemas);
        (*(all[num_schemas].id)) = num_schemas;
        all[num_schemas].fps = 0.;
        all[num_schemas].k =0;
        all[num_schemas].state=slept;
        all[num_schemas].terminate = NULL;
        all[num_schemas].handle = NULL;
        num_schemas++;

	myC.img=(char *)malloc (width[6]*height[6]*3);
	myC.width=width[6];
	myC.height=height[6];
	myC.clock=0;
        myexport("varcolorC","varcolorC",&myC);
        myexport("varcolorC","id",&varcolorC_schema_id);
        myexport("varcolorC","run",(void *)myvarcolorC_run);
        myexport("varcolorC","stop",(void *)myvarcolorC_stop);
  }

  /*creates new schema for varcolorD*/
  if(serve_color[7]==1){
        all[num_schemas].id = (int *) &varcolorD_schema_id;
        strcpy(all[num_schemas].name,"varcolorD");
        all[num_schemas].run = (runFn) myvarcolorD_run;
        all[num_schemas].stop = (stopFn) myvarcolorD_stop;
        printf("%s schema loaded (id %d)\n",all[num_schemas].name,num_schemas);
        (*(all[num_schemas].id)) = num_schemas;
        all[num_schemas].fps = 0.;
        all[num_schemas].k =0;
        all[num_schemas].state=slept;
        all[num_schemas].terminate = NULL;
        all[num_schemas].handle = NULL;
        num_schemas++;

	myD.img=(char *)malloc (width[7]*height[7]*3);
	myD.width=width[7];
	myD.height=height[7];
	myD.clock=0;
        myexport("varcolorD","varcolorD",&myD);
        myexport("varcolorD","id",&varcolorD_schema_id);
        myexport("varcolorD","run",(void *)myvarcolorD_run);
        myexport("varcolorD","stop",(void *)myvarcolorD_stop);
  }
  
  printf("firewire22 driver started up\n");
}
